@@ -3,9 +3,11 @@ import logging
 import datetime
 
 from influxdb_client import InfluxDBClient
+from influxdb_client.client.write_api import SYNCHRONOUS
 from influxdb_client.client.flux_table import TableList
 
 from sflux.utils import parse_to_string
+from sflux.measurement import Measurement
 
 try:
     import pandas as pd
@@ -22,20 +24,45 @@ class Client(InfluxDBClient):
         self.token = token
         self.org = org
 
-    def query(self, bucket: str):
+    def query(self, bucket: str, measurement: str = None):
         """
         Equivalent to the FROM statement in FluxQL
-        :param bucket: Name of the bucket to be queried
+        :param bucket:      Name of the bucket to be queried
+        :param measurement: Optional name of the measurement to be queried
         """
-        return _Query.new(self, bucket)
+        return _Query.new(self, bucket, measurement)
 
-    def insert(self, measurements: (list, tuple)):
+    def insert(self, bucket: str, measurements: (list, tuple), write_mode: str = 'SYNCHRONOUS'):
         """
         Inserts a list of measurement objects to influx
+        :param bucket:       Bucket where the measurements will be inserted
         :param measurements: List of MEASUREMENT objects
+        :param write_mode:   Can either be SYNCHRONOUS or ASYNCHRONOUS as defined in influxdb_client
         """
+        if isinstance(measurements, Measurement):
+            measurements = [measurements]
+        if write_mode == 'SYNCHRONOUS':
+            options = {'write_options': SYNCHRONOUS}
+        else:
+            options = {'success_callback': self.on_write_success,
+                       'error_callback': self.on_write_error,
+                       'retry_callback': self.on_write_retry}
+
+        with self.write_api(**options) as write_api:
+            write_api.write(bucket=bucket, org=self.org, record='\n'.join(str(elem) for elem in measurements))
+
+    def on_write_success(self, conf: (str, str, str), data: str):
         pass
 
+    def on_write_error(self, conf: (str, str, str), data: str, exception: Exception):
+        pass
+
+    def on_write_retry(self, conf: (str, str, str), data: str, exception: Exception):
+        pass
+
+    def check_health(self):
+        return self.ping()
+
 
 def add_to_query(func):
     """
@@ -62,8 +89,11 @@ class _Query:
         self._client = client
 
     @classmethod
-    def new(cls, client: InfluxDBClient, bucket: str) -> "_Query":
-        return cls(components=[f'from(bucket: "{bucket}")'], client=client)
+    def new(cls, client: InfluxDBClient, bucket: str, measurement: str = None) -> "_Query":
+        comps = [f'from(bucket: "{bucket}")']
+        if measurement is not None:
+            comps.append(f'|> filter(fn: (r) => r["_measurement"] == "{measurement}")')
+        return cls(components=comps, client=client)
 
     #######################################
     # Query methods
@@ -107,9 +137,7 @@ class _Query:
             rows = [rows]
         if isinstance(columns, str):
             columns = [columns]
-        rows = json.dumps(rows)
-        columns = json.dumps(columns)
-        return f'|> pivot(rowKey: {rows}, columnKey: {columns}, valueColumn: "{value}")'
+        return f'|> pivot(rowKey: {parse_to_string(rows)}, columnKey: {parse_to_string(columns)}, valueColumn: "{value}")'
 
     @add_to_query
     def group(self, columns: (str, list, tuple) = None, mode: str = None) -> str:
@@ -119,8 +147,7 @@ class _Query:
         if columns is not None:
             if isinstance(columns, str):
                 columns = [columns]
-            columns = json.dumps(columns)
-            column_component = f'columns: {columns}'
+            column_component = f'columns: {parse_to_string(columns)}'
         else:
             column_component = ''
 
@@ -138,8 +165,7 @@ class _Query:
         """
         if isinstance(columns, str):
             columns = [columns]
-        columns = json.dumps(columns)
-        return f'|> group(columns: {columns}, desc: "{desc}")'
+        return f'|> sort(columns: {parse_to_string(columns)}, desc: {parse_to_string(desc)})'
 
     @add_to_query
     def limit(self, n: int = 10, offset: int = 0) -> str:
@@ -217,17 +243,14 @@ class _Query:
 
     def to_dict(self) -> list:
         """
-        Returns the results as a 2 dimensional list of dictionaries, where the first dimension represents the tables
-        and the seconds the records of each table.
+        Returns the results as a pandas friendly list of dictionaries
         """
         results = self.all()
-        tables = []
+        output = []
         for table in results:
-            records = []
             for record in table.records:
-                records.append(record.values)
-            tables.append(records)
-        return tables
+                output.append(record.values)
+        return output
 
     #######################################
     # Helper methods
