@@ -1,22 +1,25 @@
+import json
+
+
 class ROW:
     """"""
     def __init__(self, column: str):
         self.column = column
 
     def __eq__(self, other: str):
-        return f'{self} == {self._parse_other(other)}'
+        return f'{self} == {parse_to_string(other)}'
 
     def __ne__(self, other):
-        return f'{self} != {self._parse_other(other)}'
+        return f'{self} != {parse_to_string(other)}'
 
     def __ge__(self, other):
-        return f'{self} >= {self._parse_other(other)}'
+        return f'{self} >= {parse_to_string(other)}'
 
     def __gt__(self, other):
-        return f'{self} > {self._parse_other(other)}'
+        return f'{self} > {parse_to_string(other)}'
 
     def __le__(self, other):
-        return f'{self} <= {self._parse_other(other)}'
+        return f'{self} <= {parse_to_string(other)}'
 
     def __lt__(self, other):
         return f'{self} < {parse_to_string(other)}'
@@ -81,8 +84,4 @@ def _and(*args):
 
 
 def parse_to_string(other):
-    if isinstance(other, (list, tuple)):
-        return '[' + ','.join([parse_to_string(elem) for elem in other]) + ']'
-    if isinstance(other, str):
-        return f'"{other}"'
-    return str(other)
\ No newline at end of file
+    return json.dumps(other)
